@@ -6,6 +6,8 @@
 #
 # Copyright 2014-2016 VeriSign, Inc.
 #
+# Copyright 2016-2017 Casey Deccio.
+#
 # DNSViz is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
@@ -41,7 +43,7 @@ import dns.exception, dns.name
 
 from dnsviz.analysis import OfflineDomainNameAnalysis, DNS_RAW_VERSION
 from dnsviz.format import latin1_binary_to_string as lb2s
-from dnsviz.util import TRUSTED_KEYS_ROOT, get_trusted_keys
+from dnsviz.util import get_trusted_keys
 
 # If the import of DNSAuthGraph fails because of the lack of pygraphviz, it
 # will be reported later
@@ -316,7 +318,8 @@ def main(argv):
             if name_str not in analysis_structured or analysis_structured[name_str].get('stub', True):
                 logger.error('The analysis of "%s" was not found in the input.' % lb2s(name.to_text()))
                 continue
-            name_objs.append(OfflineDomainNameAnalysis.deserialize(name, analysis_structured, cache))
+            name_obj = OfflineDomainNameAnalysis.deserialize(name, analysis_structured, cache)
+            name_objs.append(name_obj)
 
         if not name_objs:
             sys.exit(4)
