@@ -11,6 +11,8 @@
 #
 # Copyright 2014-2016 VeriSign, Inc.
 #
+# Copyright 2016-2017 Casey Deccio.
+#
 # DNSViz is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
@@ -771,8 +773,9 @@ class DNSQueryHandler:
                 # Explicitly specifying a client IP that cannot connect to a
                 # given destination (e.g., because it is of the wrong address
                 # scope) will result in a regular network failure with
-                # EHOSTUNREACH, as there is no scope comparison in this code.)
-                if retry_action.cause == RETRY_CAUSE_NETWORK_ERROR and retry_action.cause_arg in (errno.EHOSTUNREACH, errno.ENETUNREACH) and client is None:
+                # EHOSTUNREACH or ENETUNREACH, as there is no scope comparison
+                # in this code.)
+                if retry_action.cause == RETRY_CAUSE_NETWORK_ERROR and retry_action.cause_arg in (errno.EHOSTUNREACH, errno.ENETUNREACH, errno.EAFNOSUPPORT) and client is None:
                     raise AcceptResponse
 
                 # if this error was our fault, don't add it to the history
@@ -1430,11 +1433,14 @@ class ExecutableDNSQuery(DNSQuery):
                             raise PortBindError('Unable to bind to local port %d (%s)' % (qh.params['sport'], errno.errorcode[errno1]))
                         else:
                             raise PortBindError('Unable to bind to local port (%s)' % (errno.errorcode[errno1]))
-                    elif qtm.src is None and errno1 not in (errno.EHOSTUNREACH, errno.ENETUNREACH):
-                        # If source is None it didn't bind properly.  If errno1
-                        # is also EHOSTUNREACH, it is because there was no
-                        # proper IPv4 or IPv6 connectivity (which is handled
-                        # elsewhere); otherwise, it was something unknown, so
+                    elif qtm.src is None and errno1 not in (errno.EHOSTUNREACH, errno.ENETUNREACH, errno.EAFNOSUPPORT):
+                        # If source is None it didn't bind properly.  If the
+                        # errno1 value after bind() is EHOSTUNREACH or
+                        # ENETUNREACH, it is because there was no proper IPv4
+                        # or IPv6 connectivity (which is handled elsewhere).
+                        # If socket() failed and resulted in an errno value of
+                        # EAFNOSUPPORT, then likewise there is not IPv6
+                        # support. In other cases, it was something unknown, so
                         # raise an error.
                         raise BindError('Unable to bind to local address (%s)' % (errno.errorcode.get(errno1, "unknown")))
 
