@@ -5,6 +5,8 @@
 #
 # Copyright 2014-2016 VeriSign, Inc.
 #
+# Copyright 2016 Casey Deccio.
+#
 # DNSViz is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
@@ -556,7 +558,7 @@ class DNSQueryTransportHandlerMulti(DNSQueryTransportHandler):
 
         # load the json content
         try:
-            content = json.loads(lb2s(self.res))
+            content = json.loads(codecs.decode(self.res, 'utf-8'))
         except ValueError:
             raise RemoteQueryTransportError('JSON decoding of response failed: %s' % self.res)
 
@@ -658,7 +660,7 @@ class DNSQueryTransportHandlerHTTP(DNSQueryTransportHandlerMulti):
 
     def init_req(self):
         data = self._post_data()
-        self.req = codecs.encode('POST %s HTTP/1.1\r\nHost: %s\r\nUser-Agent: DNSViz/0.6.4\r\nAccept: application/json\r\n%sContent-Length: %d\r\nContent-Type: application/x-www-form-urlencoded\r\n\r\n%s' % (self.path, self.host, self._authentication_header(), len(data), data), 'latin1')
+        self.req = codecs.encode('POST %s HTTP/1.1\r\nHost: %s\r\nUser-Agent: DNSViz/0.6.6\r\nAccept: application/json\r\n%sContent-Length: %d\r\nContent-Type: application/x-www-form-urlencoded\r\n\r\n%s' % (self.path, self.host, self._authentication_header(), len(data), data), 'latin1')
         self.req_len = len(self.req)
         self.req_index = 0
 
@@ -840,6 +842,12 @@ class DNSQueryTransportHandlerWebSocket(DNSQueryTransportHandlerMulti):
         return val
 
     def finalize(self):
+        # python3/python2 dual compatibility
+        if isinstance(self.msg_recv, str):
+            decode_func = lambda x: struct.unpack(b'!B', x)[0]
+        else:
+            decode_func = lambda x: x
+
         new_res = b''
         for i, mask_index in enumerate(self.mask_mapping):
             mask_octets = struct.unpack(b'!BBBB', self.res[mask_index:mask_index + 4])
@@ -848,14 +856,14 @@ class DNSQueryTransportHandlerWebSocket(DNSQueryTransportHandlerMulti):
             else:
                 buf = self.res[mask_index + 4:self.mask_mapping[i + 1]]
             for j in range(len(buf)):
-                b = struct.unpack(b'!B', buf[j])[0]
+                b = decode_func(buf[j])
                 new_res += struct.pack(b'!B', b ^ mask_octets[j % 4]);
         self.res = new_res
 
         super(DNSQueryTransportHandlerWebSocket, self).finalize()
 
     def init_req(self):
-        data = json.dumps(self.serialize_requests())
+        data = codecs.encode(json.dumps(self.serialize_requests()), 'utf-8')
 
         header = b'\x81'
         l = len(data)
@@ -864,7 +872,7 @@ class DNSQueryTransportHandlerWebSocket(DNSQueryTransportHandlerMulti):
         elif l <= 0xffff:
             header += struct.pack(b'!BH', 126, l)
         else: # 0xffff < len <= 2^63
-            header += struct.pack(b'!BL', 127, l)
+            header += struct.pack(b'!BLL', 127, 0, l)
         self.req = header + data
         self.req_len = len(self.req)
         self.req_index = 0
@@ -913,7 +921,7 @@ class DNSQueryTransportHandlerWebSocket(DNSQueryTransportHandlerMulti):
                                 self.res_len = byte1b
                             elif byte1b == 126:
                                 self.res_len = struct.unpack(b'!H', self.res_buf[2:4])[0]
-                            elif byte1b == 127:
+                            else: # byte1b == 127:
                                 self.res_len = struct.unpack(b'!Q', self.res_buf[2:10])[0]
 
                             # handle mask
